@@ -134,7 +134,7 @@ thankyou_page () {
 
 	echo "
 		<big-red>
-			Thankyou for using this service
+			Thankyou for using this service.<br>Please click Continue for access.
 		</big-red>
 		<br>
 		<b>Welcome</b>
@@ -189,6 +189,9 @@ landing_page() {
 	originurl=$(printf "${originurl//%/\\x}")
 	gatewayurl=$(printf "${gatewayurl//%/\\x}")
 
+	configure_log_location
+	. $mountpoint/ndscids/ndsinfo
+
 	# authenticate and write to the log - returns with $ndsstatus set
 	auth_log
 
@@ -227,7 +230,7 @@ landing_page() {
 			<img style=\"width:100%; max-width: 100%;\" src=\"$banner3\" alt=\"Placeholder: Banner1.\"><br>
 			<b>$banner3_message</b><br>
 		</p>
-		<hr>
+
 		<p>
 			<italic-black>
 				Your login attempt probably timed out.
@@ -238,7 +241,7 @@ landing_page() {
 			Click or tap Continue to try again.
 		</p>
 		<form>
-			<input type=\"button\" VALUE=\"Continue\" onClick=\"location.href='$originurl'\" >
+			<input type=\"button\" VALUE=\"Continue\" onClick=\"location.href='http://$gatewayfqdn'\" >
 		</form>
 		<hr>
 	"
