@@ -19,7 +19,7 @@
 \********************************************************************/
 
 /** @file fw_iptables.h
-    @brief Firewall iptables functions
+    @brief Firewall nftables functions
     @author Copyright (C) 2004 Philippe April <papril777@yahoo.com>
     @author Copyright (C) 2007 Paul Kube <nodogsplash@kokoro.ucsd.edu>
     @author Copyright (C) 2015-2023 Modifications and additions by BlueWave Projects and Services <opennds@blue-wave.net>
@@ -33,20 +33,19 @@
 
 /*@{*/
 /**Iptable chain names used by opennds */
-#define CHAIN_FORWARD "FORWARD"
+#define CHAIN_PREROUTING "ndsPRE"
+#define CHAIN_POSTROUTING "ndsPOST"
+#define CHAIN_FORWARD "ndsFWD"
 #define CHAIN_FORWARD_ALLOW "nds_allow_FWD"
-#define CHAIN_INPUT "INPUT"
+#define CHAIN_INPUT "ndsINP"
 #define CHAIN_INPUT_ALLOW "nds_allow_INP"
 #define CHAIN_TO_INTERNET "ndsNET"
 #define CHAIN_TO_ROUTER "ndsRTR"
-#define CHAIN_TRUSTED_TO_ROUTER "ndsTRT"
 #define CHAIN_OUTGOING  "ndsOUT"
 #define CHAIN_INCOMING  "ndsINC"
 #define CHAIN_UPLOAD_RATE  "ndsULR"
+#define CHAIN_DOWNLOAD_RATE  "ndsDLR"
 #define CHAIN_AUTHENTICATED     "ndsAUT"
-#define CHAIN_PREAUTHENTICATED   "ndsPRE"
-#define CHAIN_BLOCKED    "ndsBLK"
-#define CHAIN_ALLOWED    "ndsALW"
 #define CHAIN_TRUSTED    "ndsTRU"
 /*@}*/
 
@@ -54,9 +53,8 @@
 /** Used to mark packets, and characterize client state.  Unmarked packets are considered 'preauthenticated' */
 extern unsigned int  FW_MARK_PREAUTHENTICATED; /**< @brief 0: Actually not used as a packet mark */
 extern unsigned int  FW_MARK_AUTHENTICATED;    /**< @brief The client is authenticated */
-extern unsigned int  FW_MARK_BLOCKED;          /**< @brief The client is blocked */
 extern unsigned int  FW_MARK_TRUSTED;          /**< @brief The client is trusted */
-extern unsigned int  FW_MARK_MASK;             /**< @brief Iptables mask: bitwise or of the others */
+extern unsigned int  FW_MARK_MASK;             /**< @brief nftables mask: bitwise or of the others */
 
 
 /** @brief Initialize the firewall */
@@ -65,8 +63,8 @@ int iptables_fw_init(void);
 /** @brief Destroy the firewall */
 int iptables_fw_destroy(void);
 
-/** @brief Helper function for iptables_fw_destroy */
-int iptables_fw_destroy_mention( const char table[], const char chain[], const char mention[]);
+/** @brief Create the client access control ruleset */
+int create_client_ruleset(char rulesetname[], char ruleset[]);
 
 /** @brief Define the access of a specific client */
 int iptables_fw_authenticate(t_client *client);
@@ -90,14 +88,8 @@ int iptables_fw_counters_update(void);
 /** @brief Return a string representing a connection state */
 const char *fw_connection_state_as_string(int mark);
 
-/** @brief Fork an iptables command */
-int iptables_do_command(const char format[], ...);
-
-int iptables_block_mac(const char mac[]);
-int iptables_unblock_mac(const char mac[]);
-
-int iptables_allow_mac(const char mac[]);
-int iptables_unallow_mac(const char mac[]);
+/** @brief Fork an nftables command */
+int nftables_do_command(const char format[], ...);
 
 int iptables_trust_mac(const char mac[]);
 int iptables_untrust_mac(const char mac[]);
