@@ -374,7 +374,10 @@ int download_remotes(int refresh)
 	int daemonpid = 0;
 	s_config *config = config_get_config();
 
-	if (config->themespec_path == NULL) {
+	// If themespec is not set then we do not need to download remotes
+	// client_params.sh does its own downloads for the 511 status pages
+
+	if (strcmp(config->themespec_path, "") == 0) {
 		return 0;
 	}
 
@@ -416,7 +419,7 @@ int write_client_info(char* msg, int msg_len, const char *mode, const char *cid,
 {
 	char *cmd = NULL;
 	s_config *config = config_get_config();
-
+	cmd = safe_calloc(MID_BUF);
 	debug(LOG_DEBUG, "Client Info: %s", info);
 	safe_asprintf(&cmd, "/usr/lib/opennds/libopennds.sh '%s' '%s' '%s' '%s'", mode, cid, config->tmpfsmountpoint, info);
 		debug(LOG_DEBUG, "WriteClientInfo command: %s", cmd);
@@ -436,10 +439,68 @@ int write_client_info(char* msg, int msg_len, const char *mode, const char *cid,
 	return 0;
 }
 
+int check_heartbeat()
+{
+	char *cmd;
+	char *msg;
+	int ret;
+
+	cmd = safe_calloc(STATUS_BUF);
+	msg = safe_calloc(STATUS_BUF);
+
+	safe_asprintf(&cmd, "/usr/lib/opennds/libopennds.sh check_heartbeat");
+
+	ret = execute_ret_url_encoded(msg, STATUS_BUF - 1, cmd);
+
+	free (cmd);
+	free (msg);
+	return ret;
+}
+
+int get_option_from_config(char* msg, int msg_len, const char *option)
+{
+	char *cmd;
+
+	cmd = safe_calloc(STATUS_BUF);
+	safe_asprintf(&cmd, "/usr/lib/opennds/libopennds.sh get_option_from_config '%s'", option);
+
+	if (execute_ret_url_encoded(msg, msg_len - 1, cmd) != 0) {
+		debug(LOG_INFO, "Failed to get option [%s] - retrying", option);
+		sleep(1);
+
+		if (execute_ret_url_encoded(msg, msg_len - 1, cmd) != 0) {
+			debug(LOG_INFO, "Failed to get option [%s] - giving up", option);
+		}
+	}
+
+	free (cmd);
+	return 0;
+}
+
+
+int get_list_from_config(char* msg, int msg_len, const char *list)
+{
+	char *cmd;
+
+	cmd = safe_calloc(MID_BUF);
+	safe_asprintf(&cmd, "/usr/lib/opennds/libopennds.sh get_list_from_config '%s'", list);
+
+	if (execute_ret_url_encoded(msg, msg_len - 1, cmd) != 0) {
+		debug(LOG_INFO, "Failed to get list [%s] - retrying", list);
+		sleep(1);
+
+		if (execute_ret_url_encoded(msg, msg_len - 1, cmd) != 0) {
+			debug(LOG_INFO, "Failed to get list [%s] - giving up", list);
+		}
+	}
+	free (cmd);
+	return 0;
+}
+
 int get_client_interface(char* clientif, int clientif_len, const char *climac)
 {
-	char *clifcmd = NULL;
-
+	char *clifcmd;
+	clifcmd = safe_calloc(SMALL_BUF);
 	safe_asprintf(&clifcmd, "/usr/lib/opennds/get_client_interface.sh %s", climac);
 
 	if (execute_ret_url_encoded(clientif, clientif_len - 1, clifcmd) == 0) {
@@ -463,7 +524,7 @@ int get_client_interface(char* clientif, int clientif_len, const char *climac)
 int hash_str(char* hash, int hash_len, const char *src)
 {
 	char *hashcmd = NULL;
-
+	hashcmd = safe_calloc(SMALL_BUF);
 	s_config *config = config_get_config();
 
 	safe_asprintf(&hashcmd, "printf '%s' | %s | awk -F' ' '{printf $1}'", src, config->fas_hid);
@@ -500,12 +561,12 @@ static int _execute_ret(char* msg, int msg_len, const char *cmd)
 
 	fp = popen(cmd, "r");
 	if (fp == NULL) {
-		debug(LOG_ERR, "popen(): [%s] Retrying..", strerror(errno));
+		debug(LOG_INFO, "popen(): [%s] Retrying..", strerror(errno));
 		sleep(1);
 		fp = popen(cmd, "r");
 
 		if (fp == NULL) {
-			debug(LOG_ERR, "popen(): [%s] Giving up..", strerror(errno));
+			debug(LOG_INFO, "popen(): [%s] Giving up..", strerror(errno));
 			rc = -1;
 			goto abort;
 		}
@@ -525,8 +586,8 @@ static int _execute_ret(char* msg, int msg_len, const char *cmd)
 	rc = pclose(fp);
 
 	if (WIFSIGNALED(rc) != 0) {
-		debug(LOG_NOTICE, "Command process exited due to signal [%d]", WTERMSIG(rc));
-		debug(LOG_NOTICE, "Requested command: [%s]", cmd);
+		debug(LOG_DEBUG, "Command process exited due to signal [%d]", WTERMSIG(rc));
+		debug(LOG_DEBUG, "Requested command: [%s]", cmd);
 		rc = WTERMSIG(rc);
 	} else {
 		rc = WEXITSTATUS(rc);
@@ -591,26 +652,36 @@ int execute_ret_url_encoded(char* msg, int msg_len, const char *cmd)
 char *
 get_iface_ip(const char ifname[], int ip6)
 {
-	char addrbuf[INET6_ADDRSTRLEN] = {0};
-	char cmd[256] = {0};
-	char iptype[8] = {0};
+	char addrbuf[STATUS_BUF] = {0};
+	char *cmd;
+	char *iptype;
+
+	iptype = safe_calloc(STATUS_BUF);
 
 	if (ip6) {
-		snprintf(iptype, sizeof(iptype), "inet6");
+		snprintf(iptype, STATUS_BUF, "inet6");
 	} else {
-		snprintf(iptype, sizeof(iptype), "inet");
+		snprintf(iptype, STATUS_BUF, "inet");
  	}
 
-	snprintf(cmd, sizeof(cmd), "/usr/lib/opennds/libopennds.sh gatewayip \"%s\" \"%s\"",
+	cmd = safe_calloc(STATUS_BUF);
+
+	snprintf(cmd, STATUS_BUF, "/usr/lib/opennds/libopennds.sh gatewayip \"%s\" \"%s\"",
 		ifname,
 		iptype
 	);
 
+	free(iptype);
+
 	debug(LOG_NOTICE, "Attempting to Bind to interface: %s", ifname);
 
-	if (execute_ret(addrbuf, sizeof(addrbuf), cmd) == 0) {
+	memset(addrbuf, 0, STATUS_BUF);
+
+	if (execute_ret(addrbuf, STATUS_BUF, cmd) == 0) {
+		free(cmd);
 		return safe_strdup(addrbuf);
 	} else {
+		free(cmd);
 		return "error";
 	}
 }
@@ -618,23 +689,30 @@ get_iface_ip(const char ifname[], int ip6)
 char *
 get_iface_mac(const char ifname[])
 {
-	char addrbuf[20] = {0};
-	char cmd[128] = {0};
+	char addrbuf[STATUS_BUF] = {0};
+	char *cmd;
 	s_config *config;
 
 	config = config_get_config();
 
+	debug(LOG_DEBUG, "Attempting to get mac of interface: %s", ifname);
+
 	if (config->gw_mac == NULL) {
 		config->gw_mac = safe_strdup("00:00:00:00:00:00");
 	}
 
-	snprintf(cmd, sizeof(cmd), "/usr/lib/opennds/libopennds.sh \"gatewaymac\" \"%s\" \"%s\"",
+	cmd = safe_calloc(STATUS_BUF);
+
+	snprintf(cmd, STATUS_BUF, "/usr/lib/opennds/libopennds.sh \"gatewaymac\" \"%s\" \"%s\"",
 		ifname,
 		config->gw_mac
 	);
 
 
-	execute_ret(addrbuf, sizeof(addrbuf), cmd);
+	memset(addrbuf, 0, STATUS_BUF);
+
+	execute_ret(addrbuf, STATUS_BUF, cmd);
+	free(cmd);
 	return safe_strdup(addrbuf);
 }
 
@@ -1024,11 +1102,13 @@ ndsctl_json_client(FILE *fp, const t_client *client, time_t now, char *indent)
 {
 	unsigned long int durationsecs;
 	unsigned long long int download_bytes, upload_bytes;
-	char clientif[64] = {0};
+	char *clientif;
 	s_config *config;
 
 	config = config_get_config();
-	get_client_interface(clientif, sizeof(clientif), client->mac);
+
+	clientif = safe_calloc(STATUS_BUF);
+	get_client_interface(clientif, STATUS_BUF, client->mac);
 
 	fprintf(fp, "  %s\"gatewayname\":\"%s\",\n", indent, config->url_encoded_gw_name);
 	fprintf(fp, "  %s\"gatewayaddress\":\"%s\",\n", indent, config->gw_address);
@@ -1046,6 +1126,8 @@ ndsctl_json_client(FILE *fp, const t_client *client, time_t now, char *indent)
 	fprintf(fp, "  %s\"clientif\":\"%s\",\n", indent, clientif);
 	fprintf(fp, "  %s\"session_start\":\"%lld\",\n", indent, (long long) client->session_start);
 
+	free(clientif);
+
 	if (client->session_end == 0) {
 		fprintf(fp, "  %s\"session_end\":\"null\",\n", indent);
 	} else {
@@ -1232,10 +1314,12 @@ void
 ndsctl_json(FILE *fp, const char *arg)
 {
 	char indent[5] = {0};
-	//if (arg && strlen(arg)) {
+
+	memset(indent, 0, 5);
+
 	debug(LOG_DEBUG, "arg [%s %d]", arg, strlen(arg));
+
 	if (strlen(arg) > 6) {
-		//snprintf(indent, sizeof(indent), "%s", "");
 		ndsctl_json_one(fp, arg, indent);
 	} else {
 		snprintf(indent, sizeof(indent), "%s", "    ");
